from json import loads
from uuid import uuid4

from django.conf import settings
from django.db import models
from django.db import transaction
from django.db.models.signals import post_delete
from wagtail.models import TranslatableMixin
from wagtail_localize.models import TranslationSource
from wagtailstreamforms import hooks
from wagtailstreamforms.models.form import AbstractForm
from wagtailstreamforms.utils.loading import get_advanced_settings_model

from . import get_translatableform_model_string, get_translatableform_model
from .utils import get_translation_source_content, get_unique_fields_names


class CustomTranslatableMixin(TranslatableMixin):
    def get_translation(self, locale):
        try:
            return super().get_translation(locale)
        except self.DoesNotExist:
            obj_copy = self.copy_for_translation(locale)
            for field in get_unique_fields_names():
                value = getattr(obj_copy, field)
                if type(value) is not str:
                    continue
                setattr(obj_copy, field, f"{value}-{str(locale)}")
                obj_copy.save()
            return super().get_translation(locale)

    class Meta:
        abstract = True
        unique_together = [("translation_key", "locale")]


class CustomFormSubmission(models.Model):
    """Data for a form submission."""

    form_data = models.TextField("Form data")
    form = models.ForeignKey(
        get_translatableform_model_string(),
        verbose_name="TranslatableForm",
        on_delete=models.CASCADE,
    )
    submit_time = models.DateTimeField("Submit time", auto_now_add=True)

    def get_data(self):
        """Returns dict with form data."""
        form_data = loads(self.form_data)
        form_data.update({"submit_time": self.submit_time})
        return form_data

    def __str__(self):
        return self.form_data

    class Meta:
        ordering = ["-submit_time"]
        verbose_name = "Form submission"


class CustomFormSubmissionFile(models.Model):
    """Data for a form submission file."""

    submission = models.ForeignKey(
        "wagtail_translatableforms.CustomFormSubmission",
        verbose_name="Submission",
        on_delete=models.CASCADE,
        related_name="files",
    )
    field = models.CharField(verbose_name="Field", max_length=255)
    file = models.FileField(verbose_name="File", upload_to="streamforms/")

    def __str__(self):
        return self.file.name

    class Meta:
        ordering = ["field", "file"]
        verbose_name = "Form submission file"

    @property
    def url(self):
        return self.file.url


def delete_file_from_storage(instance, **kwargs):
    """Cleanup deleted files from disk"""
    transaction.on_commit(lambda: instance.file.delete(False))


post_delete.connect(delete_file_from_storage, sender=CustomFormSubmissionFile)


class AbstractTranslatableForm(CustomTranslatableMixin, AbstractForm):

    class Meta:
        abstract = True
        ordering = ["title"]
        unique_together = ("translation_key", "locale")

    def copy(self):
        """Copy this form and its fields."""

        form_copy = get_translatableform_model()(
            site=self.site,
            title=self.title,
            slug=uuid4(),
            template_name=self.template_name,
            fields=self.fields,
            submit_button_text=self.submit_button_text,
            success_message=self.success_message,
            error_message=self.error_message,
            post_redirect_page=self.post_redirect_page,
            process_form_submission_hooks=self.process_form_submission_hooks,
        )
        form_copy.save()

        # additionally copy the advanced settings if they exist
        SettingsModel = get_advanced_settings_model()

        if SettingsModel:
            try:
                advanced = SettingsModel.objects.get(form=self)
                advanced.pk = None
                advanced.form = form_copy
                advanced.save()
            except SettingsModel.DoesNotExist:
                pass

        return form_copy

    def get_submission_class(self):
        """Returns submission class."""
        return CustomFormSubmission

    def delete(self, **kwargs) -> tuple[int, dict[str, int]]:
        content = get_translation_source_content()
        filtered = list(
            filter(
                lambda x: (tmp := loads(x))["pk"] == self.pk
                and tmp["translation_key"] == str(self.translation_key),
                content,
            ),
        )
        if filtered:
            related_forms = self.__class__.objects.filter(
                translation_key=self.translation_key,
            )
            with transaction.atomic():
                for ele in related_forms:
                    ele.delete()
            return None
        return super().delete(**kwargs)

    def process_form_submission(self, form, ip_addr=None):
        """Runs each hook if selected in the form."""

        for fn in hooks.get_hooks("process_form_submission"):
            if fn.__name__ in self.process_form_submission_hooks:
                fn(self, form, ip_addr)

    def get_data_fields(self):
        data_fields = super().get_data_fields()
        if getattr(settings, "WAGTAIL_TRANSLATABLEFORM_SHOW_IP", False):
            data_fields.append(("IP", "IP"))
        return data_fields

    def save(self, *args, **kwargs) -> None:
        super().save(*args, **kwargs)
        source = TranslationSource.objects.get_for_instance_or_none(self)
        if source:
            TranslationSource.update_or_create_from_instance(self)


class TranslatableForm(AbstractTranslatableForm):
    pass


import ast
import contextlib
import inspect

from django.apps import apps

from . import get_translatableform_model

def get_translation_source_content():
    translation_source_model = apps.get_model(
        app_label="wagtail_localize",
        model_name="translationsource",
    )
    return translation_source_model.objects.all().values_list(
        "content_json",
        flat=True,
    )

def get_unique_fields_names():
    model = get_translatableform_model()
    classes = inspect.getmro(model)
    fields = set()
    for class_ in classes[::-1]:
        with contextlib.suppress(TypeError):
            class_source = inspect.getsource(class_)
            class_node = ast.parse(class_source)
            for node in class_node.body[0].body:
                if not isinstance(node, ast.Assign):
                    continue
                if len(node.targets) != 1:
                    continue
                if not isinstance(node.targets[0], ast.Name):
                    continue
                if "keyword('unique', Constant(True))" in ast.dump(node.value, False):
                    fields.add(node.targets[0].id)
    return fields


from drf_spectacular.utils import extend_schema_field
from rest_framework import serializers
from wagtailstreamforms.utils.general import get_slug_from_string

from . import get_translatableform_model
from .models import CustomFormSubmission


class FrieldsDataSerializer(serializers.Serializer):
    slug = serializers.CharField()
    type = serializers.CharField()
    label = serializers.CharField()
    required = serializers.BooleanField()
    help_text = serializers.CharField()
    default_value = serializers.CharField()


class TranslatableFormSerializer(serializers.ModelSerializer):
    fields_data = serializers.SerializerMethodField()

    class Meta:
        model = get_translatableform_model()
        fields = (
            "id",
            "title",
            "slug",
            "fields_data",
            "submit_button_text",
            "success_message",
            "error_message",
            "post_redirect_page",
            "process_form_submission_hooks",
        )
        read_only_fields = fields

    @extend_schema_field(field=FrieldsDataSerializer(many=True))
    def get_fields_data(self, obj):
        fields_data = [
            {"type": ele["type"], **ele["value"]} for ele in list(obj.fields.raw_data)
        ]
        return [
            {"slug": get_slug_from_string(ele["label"]), **ele} for ele in fields_data
        ]


class TranslatableFormSubmissionSerializer(serializers.ModelSerializer):
    class Meta:
        model = CustomFormSubmission
        fields = ("form_id",)


def serialize_form(form_pk, serializer=TranslatableFormSerializer):
    if form_pk:
        form_instance = get_translatableform_model().objects.get(pk=form_pk)
        form_serializer = serializer(form_instance)
        return form_serializer.data


# Generated by Django 4.2.11 on 2024-05-22 10:02

from django.db import migrations, models
import django.db.models.deletion
import uuid
import wagtail.blocks
import wagtailstreamforms.fields
import wagtailstreamforms.streamfield


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('wagtailcore', '0089_log_entry_data_json_null_to_object'),
    ]

    operations = [
        migrations.CreateModel(
            name='CustomFormSubmission',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('form_data', models.TextField(verbose_name='Form data')),
                ('submit_time', models.DateTimeField(auto_now_add=True, verbose_name='Submit time')),
            ],
            options={
                'verbose_name': 'Form submission',
                'ordering': ['-submit_time'],
            },
        ),
        migrations.CreateModel(
            name='TranslatableForm',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('translation_key', models.UUIDField(default=uuid.uuid4, editable=False)),
                ('title', models.CharField(max_length=255, verbose_name='Title')),
                ('slug', models.SlugField(allow_unicode=True, help_text='Used to identify the form in template tags', max_length=255, unique=True, verbose_name='Slug')),
                ('template_name', models.CharField(choices=[('streamforms/form_block.html', 'Default Form Template')], max_length=255, verbose_name='Template')),
                ('fields', wagtailstreamforms.streamfield.FormFieldsStreamField([('singleline', wagtail.blocks.StructBlock([('label', wagtail.blocks.CharBlock()), ('help_text', wagtail.blocks.CharBlock(required=False)), ('required', wagtail.blocks.BooleanBlock(required=False)), ('default_value', wagtail.blocks.CharBlock(required=False))], icon='placeholder', label='Text field (single line)')), ('multiline', wagtail.blocks.StructBlock([('label', wagtail.blocks.CharBlock()), ('help_text', wagtail.blocks.CharBlock(required=False)), ('required', wagtail.blocks.BooleanBlock(required=False)), ('default_value', wagtail.blocks.CharBlock(required=False))], icon='placeholder', label='Text field (multi line)')), ('date', wagtail.blocks.StructBlock([('label', wagtail.blocks.CharBlock()), ('help_text', wagtail.blocks.CharBlock(required=False)), ('required', wagtail.blocks.BooleanBlock(required=False)), ('default_value', wagtail.blocks.CharBlock(required=False))], icon='date', label='Date field')), ('datetime', wagtail.blocks.StructBlock([('label', wagtail.blocks.CharBlock()), ('help_text', wagtail.blocks.CharBlock(required=False)), ('required', wagtail.blocks.BooleanBlock(required=False)), ('default_value', wagtail.blocks.CharBlock(required=False))], icon='time', label='Time field')), ('email', wagtail.blocks.StructBlock([('label', wagtail.blocks.CharBlock()), ('help_text', wagtail.blocks.CharBlock(required=False)), ('required', wagtail.blocks.BooleanBlock(required=False)), ('default_value', wagtail.blocks.CharBlock(required=False))], icon='mail', label='Email field')), ('url', wagtail.blocks.StructBlock([('label', wagtail.blocks.CharBlock()), ('help_text', wagtail.blocks.CharBlock(required=False)), ('required', wagtail.blocks.BooleanBlock(required=False)), ('default_value', wagtail.blocks.CharBlock(required=False))], icon='link', label='URL field')), ('number', wagtail.blocks.StructBlock([('label', wagtail.blocks.CharBlock()), ('help_text', wagtail.blocks.CharBlock(required=False)), ('required', wagtail.blocks.BooleanBlock(required=False)), ('default_value', wagtail.blocks.CharBlock(required=False))], icon='placeholder', label='Number field')), ('dropdown', wagtail.blocks.StructBlock([('label', wagtail.blocks.CharBlock()), ('help_text', wagtail.blocks.CharBlock(required=False)), ('required', wagtail.blocks.BooleanBlock(required=False)), ('empty_label', wagtail.blocks.CharBlock(required=False)), ('choices', wagtail.blocks.ListBlock(wagtail.blocks.CharBlock(label='Option')))], icon='arrow-down-big', label='Dropdown field')), ('radio', wagtail.blocks.StructBlock([('label', wagtail.blocks.CharBlock()), ('help_text', wagtail.blocks.CharBlock(required=False)), ('required', wagtail.blocks.BooleanBlock(required=False)), ('choices', wagtail.blocks.ListBlock(wagtail.blocks.CharBlock(label='Option')))], icon='radio-empty', label='Radio buttons')), ('checkboxes', wagtail.blocks.StructBlock([('label', wagtail.blocks.CharBlock()), ('help_text', wagtail.blocks.CharBlock(required=False)), ('required', wagtail.blocks.BooleanBlock(required=False)), ('choices', wagtail.blocks.ListBlock(wagtail.blocks.CharBlock(label='Option')))], icon='tick-inverse', label='Checkboxes')), ('checkbox', wagtail.blocks.StructBlock([('label', wagtail.blocks.CharBlock()), ('help_text', wagtail.blocks.CharBlock(required=False)), ('required', wagtail.blocks.BooleanBlock(required=False))], icon='tick-inverse', label='Checkbox field')), ('hidden', wagtail.blocks.StructBlock([('label', wagtail.blocks.CharBlock()), ('help_text', wagtail.blocks.CharBlock(required=False)), ('required', wagtail.blocks.BooleanBlock(required=False)), ('default_value', wagtail.blocks.CharBlock(required=False))], icon='no-view', label='Hidden field')), ('singlefile', wagtail.blocks.StructBlock([('label', wagtail.blocks.CharBlock()), ('help_text', wagtail.blocks.CharBlock(required=False)), ('required', wagtail.blocks.BooleanBlock(required=False))], icon='doc-full-inverse', label='File field')), ('multifile', wagtail.blocks.StructBlock([('label', wagtail.blocks.CharBlock()), ('help_text', wagtail.blocks.CharBlock(required=False)), ('required', wagtail.blocks.BooleanBlock(required=False))], icon='doc-full-inverse', label='Files field')), ('recaptcha', wagtail.blocks.StructBlock([('label', wagtail.blocks.CharBlock()), ('help_text', wagtail.blocks.CharBlock(required=False))], icon='success', label='ReCAPTCHA field'))], use_json_field=True, verbose_name='Fields')),
                ('submit_button_text', models.CharField(default='Submit', max_length=100, verbose_name='Submit button text')),
                ('success_message', models.CharField(blank=True, help_text='An optional success message to show when the form has been successfully submitted', max_length=255, verbose_name='Success message')),
                ('error_message', models.CharField(blank=True, help_text='An optional error message to show when the form has validation errors', max_length=255, verbose_name='Error message')),
                ('process_form_submission_hooks', wagtailstreamforms.fields.HookSelectField(blank=True, verbose_name='Submission hooks')),
                ('locale', models.ForeignKey(editable=False, on_delete=django.db.models.deletion.PROTECT, related_name='+', to='wagtailcore.locale')),
                ('post_redirect_page', models.ForeignKey(blank=True, help_text='The page to redirect to after a successful submission', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='wagtailcore.page', verbose_name='Post redirect page')),
                ('site', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='wagtailcore.site')),
            ],
            options={
                'ordering': ['title'],
                'abstract': False,
                'unique_together': {('translation_key', 'locale')},
            },
        ),
        migrations.CreateModel(
            name='CustomFormSubmissionFile',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('field', models.CharField(max_length=255, verbose_name='Field')),
                ('file', models.FileField(upload_to='streamforms/', verbose_name='File')),
                ('submission', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='files', to='wagtail_translatableforms.customformsubmission', verbose_name='Submission')),
            ],
            options={
                'verbose_name': 'Form submission file',
                'ordering': ['field', 'file'],
            },
        ),
        migrations.AddField(
            model_name='customformsubmission',
            name='form',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='wagtail_translatableforms.translatableform', verbose_name='TranslatableForm'),
        ),
    ]



import datetime

from django.contrib.admin.templatetags.admin_list import result_headers, ResultList
from django.contrib.admin.utils import (
    display_for_field,
    display_for_value,
    lookup_field,
)
from django.db import models
from django.forms.utils import flatatt
from django.utils.encoding import force_str
from django.utils.html import format_html
from django.utils.safestring import mark_safe
from django.core.exceptions import ObjectDoesNotExist
from django.template import Library
from wagtail.contrib.modeladmin.templatetags.modeladmin_tags import results
from wagtail.snippets.models import SnippetAdminURLFinder

from .. import get_translatableform_model

register = Library()


def items_for_result(view, result, request):
    """
    Generates the actual list of data.
    """
    modeladmin = view.model_admin
    url_finder_class = type(
            "_SnippetAdminURLFinder",
            (SnippetAdminURLFinder,),
            {"model": get_translatableform_model()}
        )
    url_finder = url_finder_class(request.user)
    for field_name in view.list_display:
        empty_value_display = modeladmin.get_empty_value_display(field_name)
        row_classes = ["field-%s" % field_name, "title"]
        try:
            f, attr, value = lookup_field(field_name, result, modeladmin)
        except ObjectDoesNotExist:
            result_repr = empty_value_display
        else:
            empty_value_display = getattr(
                attr, "empty_value_display", empty_value_display
            )
            if f is None or f.auto_created:
                allow_tags = getattr(attr, "allow_tags", False)
                boolean = getattr(attr, "boolean", False)
                if boolean or not value:
                    allow_tags = True
                result_repr = display_for_value(value, empty_value_display, boolean)

                # Strip HTML tags in the resulting text, except if the
                # function has an "allow_tags" attribute set to True.
                if allow_tags:
                    result_repr = mark_safe(result_repr)
                if isinstance(value, (datetime.date, datetime.time)):
                    row_classes.append("nowrap")
            else:
                if isinstance(f, models.ManyToOneRel):
                    field_val = getattr(result, f.name)
                    if field_val is None:
                        result_repr = empty_value_display
                    else:
                        result_repr = field_val
                else:
                    result_repr = display_for_field(value, f, empty_value_display)

                if isinstance(
                    f, (models.DateField, models.TimeField, models.ForeignKey)
                ):
                    row_classes.append("nowrap")
        if force_str(result_repr) == "":
            result_repr = mark_safe("&nbsp;")
        row_classes.extend(
            modeladmin.get_extra_class_names_for_field_col(result, field_name)
        )
        row_attrs = modeladmin.get_extra_attrs_for_field_col(result, field_name)
        row_attrs["class"] = " ".join(row_classes)
        row_attrs_flat = flatatt(row_attrs)
        primary_button = None
        if field_name == modeladmin.get_list_display_add_buttons(request):
            primary_button = view.button_helper.get_primary_button(result)
        if primary_button is not None and primary_button.get("url"):
            yield format_html(
                '<td{}><div class="title-wrapper"><a href="{}" title="{}">{}</a></div></td>',
                row_attrs_flat,
                url_finder.get_edit_url(result),
                primary_button.get("title", ""),
                result_repr,
            )
        else:
            yield format_html("<td{}>{}</td>", row_attrs_flat, result_repr)


def results(view, object_list, request):
    for item in object_list:
        yield ResultList(None, items_for_result(view, item, request))


@register.inclusion_tag("customforms/result_list.html", takes_context=True)
def custom_result_list(context):
    """
    Displays the headers and data list together
    """
    view = context["view"]
    object_list = context["object_list"]
    headers = list(result_headers(view))
    num_sorted_fields = 0
    for h in headers:
        if h["sortable"] and h["sorted"]:
            num_sorted_fields += 1
    context.update(
        {
            "result_headers": headers,
            "num_sorted_fields": num_sorted_fields,
            "results": list(results(view, object_list, context["request"])),
        }
    )
    return context


from django.core.exceptions import ValidationError
from wagtail import blocks
from wagtail.snippets.blocks import SnippetChooserBlock
from wagtailstreamforms.blocks import WagtailFormBlock

from . import get_translatableform_model
from .serializers import TranslatableFormSerializer


class TranslatableFormBlock(WagtailFormBlock):
    form = SnippetChooserBlock(target_model=get_translatableform_model())
    form_title = blocks.TextBlock(max_length=255, required=False)
    form_text = blocks.TextBlock(required=False)
    form_action = None

    def clean(self, value):
        result = super().clean(value)
        if not any((result.get("form_title"), result.get("form_text"))):
            msg = """Either "form title" or "form text" must be specified"""
            raise blocks.StructBlockValidationError(
                block_errors={
                    "form_title": ValidationError(msg),
                    "form_text": ValidationError(msg),
                },
            )
        return result

    def get_api_representation(self, value, context=None):
        representation = self.get_prep_value(value)
        form_pk = representation.get("form")
        if form_pk:
            form_instance = get_translatableform_model().objects.get(pk=form_pk)
            form_serializer = TranslatableFormSerializer(form_instance)
            representation["form"] = form_serializer.data
        return representation

    class Meta:
        icon = "form"


from django.contrib.admin.utils import quote
from django.urls import reverse
from wagtail.contrib.modeladmin.helpers import AdminURLHelper
from wagtail.contrib.modeladmin.helpers import ButtonHelper
from wagtail.contrib.modeladmin.options import modeladmin_register
from wagtail.snippets.models import register_snippet
from wagtail.snippets.views.snippets import (
    SnippetViewSet,
    DeleteView,
)
from wagtailstreamforms.views import SubmissionListView, SubmissionDeleteView
from wagtailstreamforms.wagtail_hooks import FormModelAdmin

from . import get_translatableform_model


class CustomFormURLHelper(AdminURLHelper):
    def get_action_url(self, action, *args, **kwargs):

        if action == "submissions":
            return reverse(
                "wagtail_translatableforms:streamforms_%s" % action, args=args, kwargs=kwargs
            )

        if action == "edit":
            return reverse(
                get_translatableform_model().snippet_viewset.get_url_name("edit"),
                args=[quote(args[0])],
            )

        if action == "delete":
            return reverse(
                get_translatableform_model().snippet_viewset.get_url_name("delete"),
                args=[quote(quote(args[0]))],
            )

        if action == "create":
            return reverse(
                f"{get_translatableform_model().snippet_viewset.get_admin_url_namespace()}:list",
            )

        return super().get_action_url(action, *args, **kwargs)


class CustomFormButtonHelper(ButtonHelper):
    def button(self, pk, action, label, title, classnames_add, classnames_exclude):
        cn = self.finalise_classname(classnames_add, classnames_exclude)
        button = {
            "url": self.url_helper.get_action_url(action, quote(pk)),
            "label": label,
            "classname": cn,
            "title": title,
        }
        return button

    def get_buttons_for_obj(
        self, obj, exclude=None, classnames_add=None, classnames_exclude=None
    ):
        buttons = super().get_buttons_for_obj(obj, exclude, classnames_add, classnames_exclude)
        pk = getattr(obj, self.opts.pk.attname)
        buttons.append(
            self.button(
                pk,
                "submissions",
                "Submissions",
                "Submissions of this form",
                classnames_add,
                classnames_exclude,
            ),
        )
        return buttons

    def add_button(self, classnames_add=None, classnames_exclude=None):
        result = super().add_button(classnames_add, classnames_exclude)
        result["label"] = "Go to forms"
        result["title"] = f"Go to {self.verbose_name} creation menu"
        return result


@modeladmin_register
class CustomFormModelAdmin(FormModelAdmin):
    model = get_translatableform_model()
    menu_label = "Forms submitions"
    menu_order = 300
    button_helper_class = CustomFormButtonHelper
    url_helper_class = CustomFormURLHelper
    index_template_name = "customforms/index.html"


class CustomSubmissionListView(SubmissionListView):
    template_name = "customforms/index_submissions.html"
    model = get_translatableform_model()


class CustomSubmissionDeleteView(SubmissionDeleteView):
    model = get_translatableform_model()
    template_name = "customforms/confirm_delete.html"

    def get_success_url(self):
        return reverse(
            "wagtail_translatableforms:streamforms_submissions",
            kwargs={"pk": self.object.pk}
            )


class CustomDeleteView(DeleteView):

    def get_success_url(self):
        if "wagtail_translatableforms" in self.request.META.get("HTTP_REFERER"):
            return reverse(
                "wagtail_translatableforms_translatableform_modeladmin_index"
                )
        return super().get_success_url()


class TranslatableFromViewSet(SnippetViewSet):
    model = get_translatableform_model()
    icon = "form"
    menu_label = "Forms"
    menu_name = "Translatable forms"
    menu_order = 300
    add_to_admin_menu = True
    delete_view_class = CustomDeleteView


register_snippet(TranslatableFromViewSet)



from django.urls import path

from .views import CustomSubmissionListView, CustomSubmissionDeleteView

urlpatterns = [
    path(
        "<int:pk>/submissions/",
        CustomSubmissionListView.as_view(),
        name="streamforms_submissions",
    ),
    path(
        "<int:pk>/submissions/delete/",
        CustomSubmissionDeleteView.as_view(),
        name="streamforms_delete_submissions",
    ),
]



from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

setattr(settings, "WAGTAILSTREAMFORMS_ADMIN_MENU_LABEL", "Deprecated")
setattr(settings, "WAGTAILSTREAMFORMS_ENABLE_BUILTIN_HOOKS", False)

def get_translatableform_model_string():
    return getattr(
        settings,
        "WAGTAIL_TRANSLATABLEFORM_FORM_MODEL",
        "wagtail_translatableforms.TranslatableForm",
        )

def get_translatableform_model():

    from django.apps import apps

    model_string = get_translatableform_model_string()
    try:
        return apps.get_model(model_string, require_ready=False)
    except ValueError:
        raise ImproperlyConfigured(
            "WAGTAIL_TRANSLATABLEFORM_FORM_MODEL must be of the form 'app_label.model_name'"
        )
    except LookupError:
        raise ImproperlyConfigured(
            "WAGTAIL_TRANSLATABLEFORM_FORM_MODEL refers to model '%s' that has not been installed"
            % model_string
        )



from django.apps import AppConfig


class StreamsConfig(AppConfig):
    name = "wagtail_translatableforms"

    def ready(self) -> None:

        from django.conf import settings
        from django.core.exceptions import ImproperlyConfigured

        for app in (
            'django.contrib.admin',
            'rest_framework',
            'drf_spectacular',
            'wagtail_modeladmin',
            'wagtail.snippets',
            'wagtail',
            'wagtail_localize',
            'wagtailstreamforms',
            ):
            if app not in settings.INSTALLED_APPS:
                raise ImproperlyConfigured(
                    f"""{app} must be in installed apps in suggested order before 'wagtail_translatableforms'.
                    Make sure 'wagtail_translatableforms' goes after:
                            'django.contrib.admin',
                            'rest_framework',
                            'drf_spectacular',
                            'wagtail_modeladmin',
                            'wagtail.snippets',
                            'wagtail',
                            'wagtail_localize',
                            'wagtailstreamforms',
                """)

        super().ready()

        from json import dumps, loads

        from django.db import transaction
        from django.template.defaultfilters import pluralize
        from django.urls import include, path
        from wagtail import hooks
        from wagtailstreamforms.hooks import register
        from wagtailstreamforms.serializers import FormSubmissionSerializer

        from . import get_translatableform_model
        from .models import CustomFormSubmissionFile
        from .urls import urlpatterns
        from .utils import get_translation_source_content


        @register('process_form_submission')
        def save_customform_submission_data(instance, form, request):
            """ saves the form submission data """

            # copy the cleaned_data so we dont mess with the original
            submission_data = form.cleaned_data.copy()
            submission_data["IP"] = request.headers.get("X-Real-Ip", "-")
            # change the submission data to a count of the files
            for field in form.files.keys():
                count = len(form.files.getlist(field))
                submission_data[field] = '{} file{}'.format(count, pluralize(count))

            # save the submission data
            submission = instance.get_submission_class().objects.create(
                form_data=dumps(submission_data, cls=FormSubmissionSerializer),
                form=instance
            )

            # save the form files
            for field in form.files:
                for file in form.files.getlist(field):
                    CustomFormSubmissionFile.objects.create(
                        submission=submission,
                        field=field,
                        file=file
                    )

        @hooks.register("register_admin_urls")
        def register_admin_urls():
            return [path("wagtail_translatableforms/",
                         include((urlpatterns, "wagtail_translatableforms"),
                    ))]


        @hooks.register("construct_main_menu")
        def remove_streamform_menu_item(request, menu_items):
            menu_items[:] = list(
                filter(
                    lambda x: x.name
                    != settings.WAGTAILSTREAMFORMS_ADMIN_MENU_LABEL.lower(),
                    menu_items,
                ),
            )

        @hooks.register("before_delete_snippet")
        def delete_related_translatebleform(request, instances):
            if request.method == "POST":
                content = get_translation_source_content()
                for instance in instances:
                    if isinstance(instance, get_translatableform_model()):
                        filtered = list(
                            filter(
                                lambda x: (tmp := loads(x))["pk"] == instance.pk
                                and tmp["translation_key"]
                                == str(instance.translation_key),
                                content,
                            ),
                        )
                        if filtered:
                            related_snippets = instance.__class__.objects.filter(
                                translation_key=instance.translation_key,
                            )
                            with transaction.atomic():
                                for ele in related_snippets:
                                    ele.delete()